//CLK2(Hilo Clock/MHz)	CLK1(Comp Clock/MHz)	CLK0(RDIMM Clock/MHz)
//150	                150	                150                                           
//                                           
`timescale 1ns/1ps
module i2c_read_reg (
    input             clk         ,
    output reg [3:0]  wb_adr_i    ,
    output reg        wb_rd_i     ,
    output reg        wb_we_i     ,
    output reg [31:0] wb_dat_i    ,
    input      [31:0] wb_dat_o    ,
    input             wb_inta_o   ,    //I2C Master Interrupt  
    input             rst_n       ,
	 input             start_enable,
	 output reg [15:0] state_data  ,
    output reg [31:0] read_data            
);


reg [2:0]   cfg_step;    //chip configuration step 
reg [16:0]  cfgdata;     //chip configuration data
reg [1:0]   cmd_step;    //0 start, 1 addr, 2 restart, 3 read data
reg [1:0]   read_wait;   // Avalon-MM slave interface of I2C master core:2 cycles of fixed read latency, 0 cycles of fixed read & write wait time
reg [22:0]  wait_counter;  
reg [1:0]   slave_num;
reg [6:0]   slave_addr;
reg         core_ini;
reg [3:0]   state;
reg [31:0]  S_read_data;

localparam SLAVE_LENGTH = 2'd3; //4 devices
localparam SLAVE1 =2'd0 ;  //slave device 1
localparam SLAVE2 =2'd1 ;  //slave device 2
localparam SLAVE3 =2'd2 ;  //slave device 3
localparam SLAVE4 =2'd3 ;  //slave device 4

localparam STEP_LENGTH = 3'd6;
localparam STEP1 =3'd0 ;  //i2c core initial
localparam STEP2 =3'd1 ;  //i2c core initial
localparam STEP3 =3'd2 ;  //i2c core initial
localparam STEP4 =3'd3 ;  //i2c core initial
localparam STEP5 =3'd4 ;  //i2c core initial
localparam STEP6 =3'd5 ;  //i2c core initial
localparam STEP7 =3'd6 ; //read chip register 


localparam IDLE           = 4'd0;
localparam READ_TX_READY  = 4'd1;
localparam TX_READY_JUDGE = 4'd2;
localparam WRITE_CMD      = 4'd3;
localparam WAIT_ACK       = 4'd4;
localparam ACK_JUDGE      = 4'd5;
localparam CLEAR_NACK     = 4'd6;
localparam NEXT_STEP      = 4'd7;
localparam NEXT_STEP_WAIT = 4'd8;
localparam READ_RX_READY  = 4'd9;
localparam RX_READY_JUDGE = 4'd10;
localparam READ_CMD       = 4'd11;
localparam READ_STEP      = 4'd12;
localparam DONE           = 4'd13; 

always @(posedge clk) begin
  case(slave_num)
    default: slave_addr <= 7'h4c; 
	 SLAVE1 : slave_addr <= 7'h4c; //FPGA core T
    SLAVE2 : slave_addr <= 7'h4d; //F Tile
	 SLAVE3 : slave_addr <= 7'h4e; //R Tile
    SLAVE4 : slave_addr <= 7'h4f; //corner areas of FPGA
  endcase
end

always @(posedge clk) begin
  case(cfg_step)
    default: cfgdata <= {8'h02, 8'h00, 1'b0}; //Enable register access, address offset, value, configuration register flag  // write CTRL reg with 0x00
    STEP1 : cfgdata <= {8'h02, 8'h00, 1'b0};  // write CTRL reg with 0x00
    STEP2 : cfgdata <= {8'h03, 8'h1f, 1'b0};  // write ISER reg with 0x1f
    STEP3 : cfgdata <= {8'h08, 8'd249, 1'b0}; // write SCL low reg with 0xf9
    STEP4 : cfgdata <= {8'h09, 8'd249, 1'b0}; // write SCL high reg with 0xf9
    STEP5 : cfgdata <= {8'h0a, 8'd125, 1'b0}; // write SDA hold reg with 0xcd
    STEP6 : cfgdata <= {8'h02, 8'h01, 1'b0};  // write CTRL reg with 0x01
    STEP7 : cfgdata <= {8'h01, 8'h00, 1'b1};  // read register 01 	
  endcase
end

always @(posedge clk or negedge rst_n) begin
  if (~rst_n) begin
    state <= IDLE;
	 wb_adr_i <= 4'd0; 
    wb_dat_i <= 32'd0;
    wb_we_i  <= 1'b0;
    wb_rd_i  <= 1'b0;
    cfg_step <= 3'd0;
    slave_num<= 2'd0;
    cmd_step <= 2'd0;
    read_wait <= 2'd3;
    S_read_data <= 32'd0;
	 read_data <= 32'd0;
	 core_ini  <= 1'b1;
    wait_counter <= 23'd0;
  end
  else begin
    case (state)
      IDLE: begin
        wb_adr_i <= 4'd0; 
        wb_dat_i <= 32'd0;
        wb_we_i  <= 1'b0;
        wb_rd_i  <= 1'b0;
		  slave_num<= 2'd0;
        cmd_step <= 2'd0;
        read_wait <= 2'd3;
		  wait_counter <= 23'd0;
        if (start_enable == 1'b1) begin
          state <= READ_TX_READY;
			 if (core_ini == 1'b1) begin
			   S_read_data <= 32'd0;
				read_data <= 32'd0;
			   cfg_step <= 3'd0;
			 end
			 else
			   cfg_step <= 3'd6;
			end
      end
      
      READ_TX_READY: begin                   //READ_TX_READY  = 4'd1;
        if (cfgdata[0] == 1'b1) begin        
          wb_adr_i <= 4'd4;
          wb_we_i = 1'b0;
          if (wb_inta_o) begin
            wb_rd_i = 1'b1;
            state <= TX_READY_JUDGE;
            read_wait <= 2'd3;
          end
        end
        else begin                             //Step1 ~ Step6
          state <= WRITE_CMD;
          wb_rd_i  <= 1'b0;
          wb_dat_i <= {24'd0, cfgdata[8:1]};   //data
          wb_adr_i <= cfgdata[12:9];           //address
          wb_we_i  <= 1'b0;
        end
      end
      
      TX_READY_JUDGE: begin                 //TX_READY_JUDGE = 4'd2;
        read_wait <= read_wait - 1'b1;
        if (read_wait == 2'd1) begin
          if (wb_dat_o[0] == 1'b1) begin    //  ISR STEP7
            state <= WRITE_CMD;
            wb_rd_i  <= 1'b0;
            wb_adr_i <= 4'd0;
            wb_we_i  <= 1'b0;
            if (cmd_step == 2'd0)                    //start
    		     wb_dat_i <= {22'd0, 2'b10, slave_addr,1'b0};  //start+slave addr+write 0
            else if (cmd_step == 2'd1)               //addr
              wb_dat_i <= {23'd0, 1'b0, cfgdata[16:9]};  //register address 
			   else if (cmd_step == 2'd2)               //addr
              wb_dat_i <= {22'd0, 2'b10, slave_addr,1'b1};  // start + slave address + read control
			   else 
 			     wb_dat_i <= {22'd0, 2'b01, 8'h00};  //stop
          end
        else
            state <= READ_TX_READY;
        end       
      end
      
      WRITE_CMD: begin                  //WRITE_CMD      = 4'd3;
        wb_we_i  <= 1'b1;
        if (cfgdata[0] == 1'b1) 
		    if(cmd_step == 2'd3)
			   state <= READ_RX_READY;
			 else
            state <= WAIT_ACK;
        else
          state <= NEXT_STEP;
      end
      
      WAIT_ACK: begin                   //WAIT_ACK       = 4'd4;
        wb_we_i  <= 1'b0;
        wb_adr_i <= 4'd4;
        if (wb_inta_o) begin  //interrup asserted, begin read
          wb_rd_i = 1'b1;
          state <= ACK_JUDGE;
          read_wait <= 2'd3;
        end       
      end

      ACK_JUDGE: begin                   //ACK_JUDGE      = 4'd5;
        read_wait <= read_wait - 1'b1;
        if (read_wait == 2'd1) begin
          if (~wb_dat_o[2]) begin              //wait ACK
            cmd_step <= cmd_step + 1'b1;
            if (cmd_step == 2'd3)
              state <= READ_RX_READY;
            else
              state <= READ_TX_READY;     
           end
         else begin
            state <= CLEAR_NACK;
            wb_we_i  <= 1'b1;
            wb_rd_i  <= 1'b0;
            wb_dat_i <= 32'h1c;
            wb_adr_i <= 4'd4;
          end
        end
      end
      
      CLEAR_NACK: begin            //CLEAR_NACK     = 4'd6;
        wb_we_i  <= 1'b0;
        state    <= READ_TX_READY;
      end
      
      NEXT_STEP: begin            //NEXT_STEP      = 4'd7;
        wb_we_i  <= 1'b0;
        wb_rd_i  <= 1'b0;
        wb_dat_i <= 32'd0;
        wb_adr_i <= 4'd0;
        cmd_step <= 2'd0;
        wait_counter <= 23'd0;
        if (cfg_step == STEP_LENGTH) begin 
          S_read_data <= S_read_data<<8;
			 slave_num <= slave_num + 1'b1;
			 if (slave_num == SLAVE_LENGTH) begin
			   state <= DONE;
				read_data <= S_read_data;
			 end
			 else
		      state <= NEXT_STEP_WAIT;		    
		  end
        else  begin
          state <= NEXT_STEP_WAIT;
          cfg_step <= cfg_step + 1'b1;
		  end
      end
      
      NEXT_STEP_WAIT: begin                          
        if (wait_counter[22]) //for simulation       
          state <= READ_TX_READY;
        else
          wait_counter <= wait_counter + 1'b1;
      end

      READ_RX_READY: begin                   //READ_TX_READY  = 4'd1;
        wb_adr_i <= 4'd4;
        wb_we_i = 1'b0;	
        if (wb_inta_o) begin  	
            wb_rd_i = 1'b1;
            state <= RX_READY_JUDGE;
            read_wait <= 2'd3;
        end
      end
      
      RX_READY_JUDGE: begin                 //TX_READY_JUDGE = 4'd2;
        read_wait <= read_wait - 1'b1;
        if (read_wait == 2'd1) begin
          if (wb_dat_o[1] == 1'b1) begin 
	         cmd_step <= cmd_step + 1'b1;		 
            state <= READ_CMD;
            wb_rd_i  <= 1'b0;
            wb_adr_i <= 4'd1;
            wb_we_i  <= 1'b0;
			   wb_dat_i <= 32'd0;
	       end
          else
            state <= READ_RX_READY; 
         end		  
       end
	  
     READ_CMD: begin  	 
        if (wb_inta_o) begin  	
            wb_rd_i = 1'b1;
            state <= READ_STEP;
            read_wait <= 2'd3;
        end
      end 
		
	 READ_STEP: begin
        read_wait <= read_wait - 1'b1;
        if (read_wait == 2'd1) begin
		    S_read_data[7:0] <= wb_dat_o [7:0];
          wb_rd_i  <= 1'b0;
          wb_adr_i <= 4'd0;
			 state <= NEXT_STEP;
        end
     end
	  
     DONE: begin
		  if (wait_counter[22]) begin
		    core_ini  <= 1'b0;
          state <= IDLE;
		  end
		  else
          wait_counter <= wait_counter + 1'b1;
     end
      
     default:begin
		  core_ini  <= 1'b1;
        state <= IDLE;
     end 
    endcase
  end
end

always @(posedge clk)
  state_data <= {5'b01010,cfg_step,slave_num, cmd_step,  state};
  
endmodule
