// INTEL CONFIDENTIAL
// Copyright 2019 Intel Corporation.  All Rights Reserved.
//
// The source code contained or described herein and all documents related
// to the source code ("Material") are owned by Intel Corporation or its
// suppliers or licensors. Title to the Material remains with Intel
// Corporation or its suppliers and licensors. The Material contains trade
// secrets and proprietary and confidential information of Intel or its
// suppliers and licensors. The Material is protected by worldwide copyright
// and trade secret laws and treaty provisions. No part of the Material may
// be used, copied, reproduced, modified, published, uploaded, posted,
// transmitted, distributed, or disclosed in any way without Intel's prior
// express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or
// delivery of the Materials, either expressly, by implication, inducement,
// estoppel or otherwise. Any license under such intellectual property rights
// must be express and approved by Intel in writing.

`timescale 1ns / 100ps

module  max_load_monitor
        (
        input   wire  i_reset_n        , // reset
        input   wire  i_system_clk     , // clock
        input   wire  i_clk_div_1024_pulse,// 
        input   wire  i_cpu_resetn_gui , // CPU reset via GUI
        input   wire  i_pgm_config     , // FPGA reconfiguration
        input   wire  i_pfl_granted    , // PFL Flash access granting signal
        input   wire  i_fpga_conf_done , // FPGA configuration done
        output  wire  o_max_conf_donen , // MAX loading done
        output  wire  o_max_loadn      , // MAX loading indication
        output  wire  o_max_load_enable, // MAX loading enable
        output  wire  O_max_errorn     , // MAX AVST configuration failed
        output  wire  o_cpu_resetn       // CPU Reset
        );

//--------------------------------------------------
// Signal defination
//--------------------------------------------------

reg               fpga_conf_done_reg1      ; // 
reg               fpga_conf_done_reg2      ; // 
wire              fpga_conf_done_pos       ; // 
reg               max_load_enable          ; // 
reg    [18:0]     max_load_cnt             ; // 0~‭524287‬
reg               max_load_error           ; // 
reg    [4:0]      cpu_resetn_cnt           ; // 0~31
reg               cpu_resetn               ; // 

//--------------------------------------------------
// Main function body
//--------------------------------------------------

// Output assignments
assign fpga_conf_done_pos = (!fpga_conf_done_reg2) & fpga_conf_done_reg1; 
assign o_cpu_resetn       = cpu_resetn;
assign o_max_load_enable  = max_load_enable;      
assign o_max_conf_donen   = !fpga_conf_done_reg2 ;
assign o_max_loadn        = (!max_load_enable) || (!i_pfl_granted);
assign O_max_errorn       = !max_load_error;

//---------------------------------------------------------------------------------------

always @(posedge i_system_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
  begin    
    fpga_conf_done_reg1 <= 1'b0;
    fpga_conf_done_reg2 <= 1'b0;
  end
  else
  begin
    fpga_conf_done_reg1 <= i_fpga_conf_done;
    fpga_conf_done_reg2 <= fpga_conf_done_reg1;
  end
end


always @(posedge i_system_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     cpu_resetn_cnt <= 5'd0;
  else if(!i_cpu_resetn_gui)
     cpu_resetn_cnt <= 5'd0;
  else if(fpga_conf_done_pos)
     cpu_resetn_cnt <= 5'd0;
  else if(cpu_resetn_cnt == 5'd31)
     cpu_resetn_cnt <= cpu_resetn_cnt;
  else
     cpu_resetn_cnt <= cpu_resetn_cnt + 1'd1;
end

always @(posedge i_system_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     cpu_resetn <= 1'b0;
  else if(cpu_resetn_cnt == 5'd0)
     cpu_resetn <= 1'b0;
  else if(cpu_resetn_cnt == 5'd31)
     cpu_resetn <= 1'b1;
  else
     cpu_resetn <= cpu_resetn;
end

//---------------------------------------------------------------------------------------
//LED control and unused output signals assignment

always @(posedge i_system_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     max_load_enable <= 1'b1;
  else if(!i_pgm_config)                                                        // MAX 10 load enable trigger
     max_load_enable <= 1'b1;
  else if(fpga_conf_done_pos)                                                   // de-assert max5 load enable once fpga configuration is done, to avoid led missing flashing for jtag download way
     max_load_enable <= 1'b0;
  //else if(max_load_cnt == 16'd65535)                                          // de-assert max5 load enable if it cannot successfully configure fpga after certain duration
  else if(max_load_cnt == 19'd524287)  // 524287 * 20.48 us = 10.7s
     max_load_enable <= 1'b0;
  else
     max_load_enable <= max_load_enable;
end

always @(posedge i_system_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     max_load_cnt <= 19'd0;
  else if(!i_pgm_config)                                                        // clear counter at the beginning of each loading round
     max_load_cnt <= 19'd0;
  else if(fpga_conf_done_pos)
     max_load_cnt <= 19'd0;
  else if(max_load_enable & i_clk_div_1024_pulse)  
     max_load_cnt <= max_load_cnt + 1'd1;
  else
     max_load_cnt <= max_load_cnt;
end

always @(posedge i_system_clk or negedge i_reset_n)
begin
  if(!i_reset_n)
     max_load_error <= 1'b0;
  else if(!i_pgm_config)
     max_load_error <= 1'b0;
  else if(fpga_conf_done_reg2)                                                  // MAX 10 won't reconfigure FPGA right after itself reconfiguration via programmer
     max_load_error <= 1'b0;                                                    // When FPGA configuration done status is active(high)
  else if(max_load_cnt == 19'd524287)                                           // Assert max_load_error once counter overflow, timeout
     max_load_error <= 1'b1;
  else
     max_load_error <= max_load_error;
end


endmodule
//------------------------------------------------------------------------------
// end of file
//------------------------------------------------------------------------------