// INTEL CONFIDENTIAL
// Copyright 2019 Intel Corporation.  All Rights Reserved.
//
// The source code contained or described herein and all documents related
// to the source code ("Material") are owned by Intel Corporation or its
// suppliers or licensors. Title to the Material remains with Intel
// Corporation or its suppliers and licensors. The Material contains trade
// secrets and proprietary and confidential information of Intel or its
// suppliers and licensors. The Material is protected by worldwide copyright
// and trade secret laws and treaty provisions. No part of the Material may
// be used, copied, reproduced, modified, published, uploaded, posted,
// transmitted, distributed, or disclosed in any way without Intel's prior
// express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or
// delivery of the Materials, either expressly, by implication, inducement,
// estoppel or otherwise. Any license under such intellectual property rights
// must be express and approved by Intel in writing.

`timescale 1ns / 100ps
   
module  max_pfl_ctrl
        (
        //Reset and Clock
        input   wire        i_reset_n           , // reset
        input   wire        i_pfl_nreset        , // PFL reset
        input   wire        i_system_clk        , // clock_factory_load
        input   wire        i_clk_div_1024_pulse, // clock

        //FPGA configuration control and status
        input   wire        i_cpu_resetn_gui    , // CPU reset via GUI
        input   wire        i_fpga_reconfign_gui, // FPGA reconfiguration via GUI
        input   wire [3:0]  i_page_sel_gui      , // Page selection via GUI
        input   wire        i_factory_load      , // Factory load selection
        input   wire        i_pgm_sel           , // Flash page selection button
        input   wire        i_pgm_config        , // FPGA reconfiguration
        input   wire        i_fpga_conf_done    , // FPGA configuration done
        input   wire        i_fpga_nstatus      , // FPGA configuration status

        //Flash interface
		inout  wire         flash_io0           , // Flash_io0.flash_io0
		inout  wire         flash_io1           , // Flash_io1.flash_io1
		inout  wire         flash_io2           , // Flash_io2.flash_io2
		inout  wire         flash_io3           , // Flash_io3.flash_io3
		output wire         flash_ncs           , // Flash_ncs.flash_ncs
		output wire         flash_sck           , // Flash_sck.flash_sck

        //FPP interface	
        output  wire        o_avst_valid        , // AVST configuration valid
		input   wire        i_avst_ready        , // AVST configuration ready
		output  wire [31:0] o_avst_data         , // AVST configuration data bus
        output  wire        o_avst_clk          , // AVST configuration clock
        output  wire        o_fpga_nconfig      , // AVST configuration config control, low active

        //MAX V loading monitor status
        output  wire        o_max_conf_donen    , // MAX loading done
        output  wire        o_max_loadn         , // MAX loading indication
        output  wire        O_max_errorn        , // MAX AVST configuration failed
        output  wire [2:0]  o_fpga_pgm          , // FPGA programming page selection
        output  wire [2:0]  o_pgm_led           , // Page selection LED indication
		output  wire        o_pgm_sel_pb_trig	, // Page_sel pushbutton
        output  wire        o_cpu_resetn          // FPGA reset after conf done
        );

//--------------------------------------------------
// Signal defination
//--------------------------------------------------

wire  [2:0]     fpga_pgm                    ;
wire            pgm_config_ctrl             ;
wire            pfl_flash_access_granted    ;
wire            max_load_enable             ;   
wire            fpga_nconfig_flash          ;

//--------------------------------------------------
// Main function body
//--------------------------------------------------
assign pgm_config_ctrl = i_pgm_config & i_fpga_reconfign_gui;
assign o_fpga_nconfig  = max_load_enable ? fpga_nconfig_flash  : 1'b1;
assign o_fpga_pgm      = fpga_pgm;

//---------------------------------------------------------------------------------------
//Programming image selection
max_pgm_sel max_pgm_sel_inst(
    .i_reset_n                (i_reset_n               ), 
    .i_sys_clk                (i_system_clk            ), 
    .i_pgm_sel_disable        (max_load_enable         ), 
    .i_page_sel_gui           (i_page_sel_gui          ), 
    .i_factory_load           (i_factory_load          ), 
    .i_pgm_sel                (i_pgm_sel               ), 
    .o_pgm_led                (o_pgm_led               ), 
    .o_pgm_sel_pb_trig        (o_pgm_sel_pb_trig	   ), 
    .o_fpga_pgm               (fpga_pgm                )  
    );


//PFL interface			 				
pfl2 pfl2_inst(
	.avst_valid               (o_avst_valid            ),
	.avst_ready               (i_avst_ready            ),
	.avst_data                (o_avst_data             ),
	.avst_clk                 (o_avst_clk              ),
	.flash_io0                (flash_io0               ),
	.flash_io1                (flash_io1               ),
	.flash_io2                (flash_io2               ),
	.flash_io3                (flash_io3               ),
	.flash_ncs                (flash_ncs               ),
	.flash_sck                (flash_sck               ),
	.fpga_conf_done           (i_fpga_conf_done        ),
	.fpga_nconfig             (fpga_nconfig_flash      ),
	.fpga_nstatus             (i_fpga_nstatus          ),
	.fpga_pgm                 (fpga_pgm                ), 
	.pfl_clk                  (i_system_clk            ),
	.pfl_flash_access_granted (pfl_flash_access_granted), //input of pfl2 ip core,if low, prevent JTAG from accessing flash and FPGA configuration
	                                                      //pull high to use pfl ip function as flash master
	.pfl_flash_access_request (pfl_flash_access_granted), //output of pfl2 ip core.
	                                                      //pfl ip core drives it high when jtag accesses flash or configures fpga
	.pfl_nreconfigure         (pgm_config_ctrl         ),  
	.pfl_nreset               (i_pfl_nreset            ) //high-enable fpga configuration; pull low when don't use PFL II IP core
	);
 					
//MAX loading status monitor and fpga reset control
max_load_monitor    max_load_monitor_inst(
    .i_reset_n                (i_reset_n               ),  
    .i_system_clk             (i_system_clk            ),  
    .i_clk_div_1024_pulse     (i_clk_div_1024_pulse    ),  
    .i_cpu_resetn_gui         (i_cpu_resetn_gui        ),  
    .i_pgm_config             (pgm_config_ctrl         ),  
    .i_pfl_granted            (pfl_flash_access_granted),  
    .i_fpga_conf_done         (i_fpga_conf_done        ),  
    .o_max_conf_donen         (o_max_conf_donen        ),  
    .o_max_loadn              (o_max_loadn             ),  
    .o_max_load_enable        (max_load_enable         ),  
    .O_max_errorn             (O_max_errorn            ),  
    .o_cpu_resetn             (o_cpu_resetn            )   
    );

endmodule
//------------------------------------------------------------------------------
// end of file
//------------------------------------------------------------------------------