`timescale 1ns/1ps
module dac_i2c_wr_reg (
    input             clk         ,
    input             rst_n       ,
//avalon interface --master
    output reg [3:0]  wb_adr_i    ,
    output reg        wb_rd_i     ,
    output reg        wb_we_i     ,
    output reg [31:0] wb_dat_i    ,
    input      [31:0] wb_dat_o    ,
//Interrupt from Avalon I2C (Master) Intel FPGA IP
    input             wb_inta_o   ,    
//other control signals
	 input             start_enable , //=1, state machine is keeping on working; =0: state machine stay in IDLE
	 input[10:0]       inc_dec_num,
	 input             inc_decn,
	 output            clr_inc_dec,
	 output reg [15:0] state_data          
);

reg [2:0]   cfg_step;       //chip configuration step
reg [16:0]  cfgdata;        //chip configuration data
reg [1:0]   cmd_step;       //0 start, 1 addr, 2 data MSDB, 3 data LSDB
reg [1:0]   read_wait;      //Avalon-MM slave interface of I2C master core:2 cycles of fixed read latency, 0 cycles of fixed read & write wait time
reg [15:0]  wait_counter;  
reg [15:0]  S_dac_data;     //data writed to DAC chip reg8h
reg         S_clr_inc_dec;  //clear dec/inc pulse number that latched here
reg         core_ini;       //if I2C core initialization is done. =1: not yet; =0 done.

reg [10:0]  S_inc_dec_num;  //dec/inc pulse number
reg         S_inc_decn;     //=1: inc pulse; =0: dec pulse
reg [3:0]   state;
reg         dac_data_update; //=1: S_dac_data has been updated for this writing
reg         S_clr_inc_dec_d1;


//-----------------------------------------------------------------------------------------------------------------
//  configuration step
//-----------------------------------------------------------------------------------------------------------------
localparam SLAVE_ADDR = 7'h48;
localparam STEP_LENGTH = 3'd7;
localparam STEP1 =7'd0 ;  //i2c core initial
localparam STEP2 =7'd1 ;  //i2c core initial
localparam STEP3 =7'd2 ;  //i2c core initial
localparam STEP4 =7'd3 ;  //i2c core initial
localparam STEP5 =7'd4 ;  //i2c core initial
localparam STEP6 =7'd5 ;  //i2c core initial
localparam STEP7 =7'd6 ;  //set chip reg4h
localparam STEP8 =7'd7 ;  //set chip reg8h


//-----------------------------------------------------------------------------------------------------------------
//  state for state machine
//-----------------------------------------------------------------------------------------------------------------
localparam IDLE           = 4'd0;
localparam READ_TX_READY  = 4'd1;
localparam TX_READY_JUDGE = 4'd2;
localparam WRITE_CMD      = 4'd3;
localparam WAIT_ACK       = 4'd4;
localparam ACK_JUDGE      = 4'd5;
localparam CLEAR_NACK     = 4'd6;
localparam NEXT_STEP      = 4'd7;
localparam NEXT_STEP_WAIT = 4'd8;
localparam DONE           = 4'd9;
 
//-----------------------------------------------------------------------------------------------------------------
//  configuration data
//-----------------------------------------------------------------------------------------------------------------
always @(posedge clk) begin
  case(cfg_step)
    default: cfgdata <= {8'h02, 8'h00, 1'b0};  //write  reg02h with 00, CTRL register    
    STEP1 :  cfgdata <= {8'h02, 8'h00, 1'b0};  //write  reg02h with 00, CTRL register  
    STEP2 :  cfgdata <= {8'h03, 8'h1f, 1'b0};  //write  reg03h with 1f, ISER register  
    STEP3 :  cfgdata <= {8'h08, 8'd249, 1'b0}; //write  reg08h with f9, SCL low count register
    STEP4 :  cfgdata <= {8'h09, 8'd249, 1'b0}; //write  reg09h with f9, SCL high count register
    STEP5 :  cfgdata <= {8'h0a, 8'd125, 1'b0}; //write  reg0ah with 7d, SCL high count register
    STEP6 :  cfgdata <= {8'h02, 8'h01, 1'b0};  //write  reg02h with 01, CTRL register 
    STEP7 :  cfgdata <= {8'h04, 8'h00, 1'b1};  //set DAC chip reg4h.bit8=1, bit0=1
    STEP8 :  cfgdata <= {8'h08, 8'h00, 1'b1};  // set DAC chip reg8h
  endcase
end
  

//-----------------------------------------------------------------------------------------------------------------
//  get dac_data for DAC chip reg8h
//----------------------------------------------------------------------------------------------------------------- 
always @(posedge clk or negedge rst_n) begin
  if (~rst_n) begin
    S_inc_dec_num   <= 11'd0;
	 S_inc_decn      <= 1'd1;
	 S_dac_data      <=16'h8000;
	 dac_data_update <= 1'b0;
  end
  else if (!S_clr_inc_dec_d1) begin
	 S_inc_dec_num <= inc_dec_num;
	 S_inc_decn    <= inc_decn;
    dac_data_update <= 1'b0;
	 S_dac_data <= S_dac_data;
  end
  else begin
    S_inc_dec_num <= S_inc_dec_num;
	 S_inc_decn    <= S_inc_decn;
	 if (S_inc_decn && !dac_data_update) begin
	   S_dac_data <= S_dac_data + {1'b0,S_inc_dec_num,4'b0000};
	   dac_data_update <= 1'b1;
	 end
	 else if (!S_inc_decn && !dac_data_update) begin
	   S_dac_data <= S_dac_data - {1'b0,S_inc_dec_num,4'b0000};
	   dac_data_update <= 1'b1;
	 end
	 else
	   S_dac_data <= S_dac_data;	 
  end
end

always @(posedge clk or negedge rst_n) begin
  if (~rst_n) 
    S_clr_inc_dec_d1 <= 1'b0;

  else 
    S_clr_inc_dec_d1 <= S_clr_inc_dec;

end

//-----------------------------------------------------------------------------------------------------------------
//  state machine
//-----------------------------------------------------------------------------------------------------------------
always @(posedge clk or negedge rst_n) begin
  if (~rst_n) begin
    state <= IDLE;
	 wb_adr_i <= 4'd0; 
    wb_dat_i <= 32'd0;
    wb_we_i  <= 1'b0;
    wb_rd_i  <= 1'b0;
    cfg_step <= 3'd0;
    cmd_step <= 2'd0;
    read_wait <= 2'd3;
	 wait_counter <= 16'd0;
	 S_clr_inc_dec <= 1'b0;
	 core_ini  <= 1'b1;
  end
  else begin
    case (state)
      IDLE: begin                          //IDLE  = 4'd0;
        wb_adr_i <= 4'd0;  
        wb_dat_i <= 32'd0;
        wb_we_i  <= 1'b0;
        wb_rd_i  <= 1'b0;
        cmd_step <= 2'd0;
        read_wait <= 2'd3;
		  S_clr_inc_dec <= 1'b0;
        if (start_enable == 1'b1) begin
          state <= READ_TX_READY;
		    if (core_ini == 1'b1) 
			 cfg_step <= 3'd0;		 
		  end
      end
      
      READ_TX_READY: begin                   //READ_TX_READY  = 4'd1;
        if (cfgdata[0] == 1'b1) begin        //Step7 ~ Step8, read ISR(0x4) and decide whether write the data 
          wb_adr_i <= 4'd4;
          wb_we_i = 1'b0;
          if (wb_inta_o) begin
            wb_rd_i = 1'b1;
            state <= TX_READY_JUDGE;
            read_wait <= 2'd3;		  
          end
        end
        else begin                             //Step1 ~ Step6
          state <= WRITE_CMD;
          wb_rd_i  <= 1'b0;
          wb_dat_i <= {24'd0, cfgdata[8:1]};   //data
          wb_adr_i <= cfgdata[12:9];           //address
          wb_we_i  <= 1'b0;
        end
      end
      
      TX_READY_JUDGE: begin                 //TX_READY_JUDGE = 4'd2;
        read_wait <= read_wait - 1'b1;
        if (read_wait == 2'd1) begin
          if (wb_dat_o[0] == 1'b1) begin                        //  read ISR 
            state <= WRITE_CMD;
            wb_rd_i  <= 1'b0;
            wb_adr_i <= 4'd0;
            wb_we_i  <= 1'b0;
            if (cmd_step == 2'd0)                               //start
              wb_dat_i <= {22'd0, 2'b10, SLAVE_ADDR,1'b0};      //write slave address, rw_d:0, 0x48:100_1000
            else if (cmd_step == 2'd1)                          //addr
              wb_dat_i <= {24'd0, cfgdata[16:9]};
            else if ((cfg_step == 3'd6) && (cmd_step == 2'd2))  //write MSDB data
              wb_dat_i <= {24'd0, 8'h01};                       //set chip reg4h.bit8=1
			   else if ((cfg_step == 3'd6) && (cmd_step == 2'd3))  //write LSDB data
              wb_dat_i <= {22'd0, 2'b01, 8'h01};                //set chip reg4h.bit0=1
			   else if ((cfg_step == 3'd7) && (cmd_step == 2'd2))  //write MSDB data
              wb_dat_i <= {24'd0, S_dac_data[15:8]};            //set chip reg8h[15:8]
			   else if ((cfg_step == 3'd7) && (cmd_step == 2'd3))  //write LSDB data
              wb_dat_i <= {22'd0, 2'b01, S_dac_data[7:0]};      //set chip reg8h[7:0] 
          end
          else
            state <= READ_TX_READY;
        end       
      end
      
      WRITE_CMD: begin                  //WRITE_CMD      = 4'd3;
        wb_we_i  <= 1'b1;
        if (cfgdata[0] == 1'b1) 
          state <= WAIT_ACK;
        else
          state <= NEXT_STEP;
      end
      
      WAIT_ACK: begin                   //WAIT_ACK       = 4'd4;
        wb_we_i  <= 1'b0;
        wb_adr_i <= 4'd4;
        if (wb_inta_o) begin            //interrup asserted, begin read
          wb_rd_i = 1'b1;
          state <= ACK_JUDGE;
          read_wait <= 2'd3;
        end       
      end

      ACK_JUDGE: begin                   //ACK_JUDGE      = 4'd5;
        read_wait <= read_wait - 1'b1;
		  if (read_wait == 2'd2 && (cfg_step == 3'd7) && (cmd_step == 2'd1)) 
		    S_clr_inc_dec <= 1'b1;
        else if (read_wait == 2'd1) begin
          if (~wb_dat_o[2]) begin              //wait ACK
            cmd_step <= cmd_step + 1'b1;
            if (cmd_step == 2'd3)
              state <= NEXT_STEP;
            else
              state <= READ_TX_READY;
		      if ((cfg_step == 3'd7) && (cmd_step == 2'd3))
		        S_clr_inc_dec <= 1'b0;
          end
          else begin
            state <= CLEAR_NACK;
            wb_we_i  <= 1'b1;
            wb_rd_i  <= 1'b0;
            wb_dat_i <= 32'h1c;
            wb_adr_i <= 4'd4;
          end
        end
      end
      
      CLEAR_NACK: begin            //CLEAR_NACK     = 4'd6;
        wb_we_i  <= 1'b0;
        state    <= READ_TX_READY;
      end
      
      NEXT_STEP: begin            //NEXT_STEP      = 4'd7;
        wb_we_i  <= 1'b0;
        wb_rd_i  <= 1'b0;
        wb_dat_i <= 32'd0;
        wb_adr_i <= 4'd0;
        cfg_step <= cfg_step + 1'b1;
        cmd_step <= 2'd0;
        wait_counter <= 16'd0;
        if (cfg_step == STEP_LENGTH)
          state <= DONE;
        else
          state <= NEXT_STEP_WAIT;
      end
      
      NEXT_STEP_WAIT: begin                          //NEXT_STEP_WAIT = 4'd8;
        if (wait_counter[15]) //for simulation       
          state <= READ_TX_READY;
        else
          wait_counter <= wait_counter + 1'b1;
      end
      
      DONE: begin                                   //DONE = 4'd9;
		  if (wait_counter[9]) begin     //2^9*20ns=10.24us
		    core_ini  <= 1'b0;
			 cfg_step <= 3'd7;
			 if (inc_dec_num == 11'd0) begin
			   state <= DONE;
				wait_counter <= 16'd0;
			 end
			 else
            state <= IDLE;
		  end
		  else
            wait_counter <= wait_counter + 1'b1;
      end
      
      default:begin
		  core_ini  <= 1'b1;
        state <= IDLE;
      end 
    endcase
  end
end

assign  clr_inc_dec = S_clr_inc_dec;

always @(posedge clk)
  state_data <= {7'd0,cmd_step, cfg_step, state};

endmodule